
CODE_0_SUCCESS = 0
CODE_1_ERROR = 1
CODE_2_LOGIN_REQUIRED = 2
CODE_3_ERROR_SECURE_TOKEN = 3
CODE_4_ERROR_USERNAME_OR_PASSWORD = 4
CODE_5_INVALID_PASSWORD = 5
CODE_6_USER_CREATION_FAILED = 6
CODE_7_SEND_SMS_FAILED = 7
CODE_8_CAPTCHA_VALIDATION_ERROR = 8
CODE_9_INVALID_PHONE_NUMBER = 9
CODE_10_INVALID_EMAIL_ADDRESS = 10
CODE_11_INVALID_USERNAME = 11
CODE_12_INVALID_CHINESE_NAME = 12
CODE_13_VCODE_VALIDATION_ERROR = 13
CODE_14_PHONE_ALREADY_EXISTS_ERROR = 14
CODE_15_USERNAME_ALREADY_EXISTS_ERROR = 15
CODE_16_EMAIL_ALREADY_EXISTS_ERROR = 16
CODE_19_MISSING_REQUIRED_PARAMS = 19
CODE_20_PERMISSION_DENIED = 20
CODE_21_INVALID_PARAMETER = 21
CODE_23_OBJECT_ALREADY_EXISTS = 23
CODE_24_USER_IS_DISABLED = 24
